#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <string>
#include "cdf.h"


#define __ZMODE__


ofstream output( "vAttr.dat" );


void ErrorHandler( const CDFstatus &stat ){
	CDFstatus errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	if( stat != CDF_OK ){
		errstat = CDFerror( stat, errmesg );
		cerr << errmesg << endl;

		exit(1);
	}

	return;
}


string datatypeInquire( const long &datatype ){
	string type;

	switch( datatype ){
	case CDF_BYTE:
		type = "CDF_BYTE";
		break;
	case CDF_CHAR:
		type = "CDF_CHAR";
		break;
	case CDF_INT1:
		type = "CDF_INT1";
		break;
	case CDF_UCHAR:
		type = "CDF_UCHAR";
		break;
	case CDF_UINT1:
		type = "CDF_UINT1";
		break;
	case CDF_INT2:
		type = "CDF_INT2";
		break;
	case CDF_UINT2:
		type = "CDF_UINT2";
		break;
	case CDF_INT4:
		type = "CDF_INT4";
		break;
	case CDF_UINT4:
		type = "CDF_UINT4";
		break;
	case CDF_REAL4:
		type = "CDF_REAL4";
		break;
	case CDF_FLOAT:
		type = "CDF_FLOAT";
		break;
	case CDF_REAL8:
		type = "CDF_REAL8";
		break;
	case CDF_DOUBLE:
		type = "CDF_DOUBLE";
		break;
	case CDF_EPOCH:
		type = "CDF_EPOCH";
		break;
	default:
		cerr << "Invalid datatype.\n";
		exit(1);
	}

	return type;
}


// show data of ordinary type
template <class T>
void outAttr( ){
	char attrName[CDF_ATTR_NAME_LEN+1];
	CDFstatus stat;
	T attr;

	stat = CDFlib( GET_, zENTRY_DATA_, &attr, 
                       ATTR_NAME_, attrName, 
								 NULL_ );
	ErrorHandler( stat );

	output.setf( ios::left );
	output.precision(2);

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName << attr << endl;
	}else{
		output << attrName << "   " << attr << endl;
	}

	return;
}


// show data of char type
void outCharAttr( ){
	CDFstatus stat;
	char attrName[CDF_ATTR_NAME_LEN+1];
	char *attr;
	long numElems;

	stat = CDFlib( GET_, ATTR_NAME_, attrName, 
                       zENTRY_NUMELEMS_, &numElems, 
								 NULL_ );
	ErrorHandler( stat );

	attr = new char[numElems+1];

	stat = CDFlib( GET_, zENTRY_DATA_, attr, 
								 NULL_ );
	ErrorHandler( stat );

	attr[numElems] = '\0';

	output.setf( ios::left );

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName << attr << endl;
	}else{
		output << attrName << "   " << attr << endl;
	}

	return;
}


// show data of Epoch type
void outEpochAttr( ){
	CDFstatus stat;
	char attrName[CDF_ATTR_NAME_LEN+1];
	double attr_epoch;
	long year, mon, day, hour, min, sec, msec;

	stat = CDFlib( GET_, zENTRY_DATA_, &attr_epoch, 
                       ATTR_NAME_, &attrName, 
								 NULL_ );
	ErrorHandler( stat );

	EPOCHbreakdown( attr_epoch, &year, &mon, &day, &hour, &min, &sec, &msec );

	output.setf( ios::left );

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName;
	}else{
		output << attrName << "   ";
	}

	char time[30];
	sprintf( time, "%04d/%02d/%02d  %02d:%02d:%02d.%04d\0", 
					 year, mon, day, hour, min, sec, msec );
	output << time << "\n";

	return;
}


//void attrInquire( const int &num ){
int attrInquire( const int &attrNum, const int &zEntryNum ){
	long scope;
	long datatype;
	CDFstatus stat, errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	stat = CDFlib( SELECT_, ATTR_, attrNum, 
								 GET_, ATTR_SCOPE_, &scope, 
								 NULL_ );
	ErrorHandler( stat );

	if( scope != VARIABLE_SCOPE ) return 0; // vAttribute Ǥʤ֤


	stat = CDFlib( SELECT_, zENTRY_, zEntryNum, 
								 GET_, zENTRY_DATATYPE_, &datatype, 
								 NULL_ );

	if( stat != CDF_OK ){
		if( stat == NO_SUCH_ENTRY ){
			return 0;
		}else{
			errstat = CDFerror( stat, errmesg );
			cerr << errmesg << endl;
			exit(1);
		}
	}

	switch( datatype ){
	case CDF_CHAR:
		outCharAttr( );
		break;
// 2 byte int  short ǤΤ狼ʤɤ褦
	case CDF_INT2:
		outAttr<short>( );
		break;
	case CDF_UINT2:
		outAttr<unsigned short>( );
		break;
	case CDF_INT4:
		outAttr<short>( );
		break;
	case CDF_UINT4:
		outAttr<unsigned short>( );
		break;
	case CDF_REAL4:
	case CDF_FLOAT:
		outAttr<float>( );
		break;
	case CDF_REAL8:
	case CDF_DOUBLE:
		outAttr<double>( );
		break;
	case CDF_EPOCH:
		outEpochAttr( );
		break;
// ʲηϡɤηȤɤФ狼ʤΤα
	case CDF_BYTE:
	case CDF_INT1:
	case CDF_UINT1:
	case CDF_UCHAR:
		cerr << "Sorry.\n"
				 << "Datatype " << datatypeInquire( datatype )
				 << " is not supported\n";
		exit(1);
	default:
		cerr << "Invalid datatype.\n";
		exit(1);
	}

	return 1;
}


int main( int argc, char* argv[] ){
// Open the file
	if( argc != 2 ){
		cerr << "Usage : cdfgetattr CDF_FILE_NAME\n";
		exit(1);
	}

	CDFid id;
	char *filename = argv[1];
	CDFstatus stat, errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	stat = CDFlib( OPEN_, CDF_, filename, &id, 
#ifdef __ZMODE__
								 SELECT_, CDF_zMODE_, zMODEon2, 
#endif
								 NULL_ );  // ե򳫤

	ErrorHandler( stat ); // Error check

	long numAttrs;
	long numzVars, numvAttrs;

	stat = CDFlib( GET_, CDF_NUMzVARS_, &numzVars, 
        							 CDF_NUMATTRS_, &numAttrs, 
        							 CDF_NUMvATTRS_, &numvAttrs, 
								 NULL_ );
	ErrorHandler( stat ); // Error check


	char YorN;
	bool getall;

	while( 1 ){
		cerr << "Get all attributes ?  (y/n)\n";
		cin >> YorN;
		if( YorN == 'y' ){
			getall = true;
			break;
		}else if( YorN == 'n' ){
			getall = false;
			break;
		}else{
			cerr << "Please answer y or n.\n";
		}
	}

	int i, j;
	char varName[CDF_VAR_NAME_LEN+1];
	bool contproc;

	output << "CDF attiribute data\n";

	for( j = 0 ; j < numzVars ; j++ ){
		stat = CDFlib( SELECT_, zVAR_, j, 
									 GET_, zVAR_NAME_, varName, 
									 NULL_ );
		ErrorHandler( stat );

		if( ! getall ){
			while( 1 ){
				cerr << "Get the attributes for " << varName << " ?  (y/n)\n";
				cin >> YorN;
				if( YorN == 'y' ){
					contproc = true;
					break;
				}else if( YorN == 'n' ){
					contproc = false;
					break;
				}else{
					cerr << "Please answer y or n.\n";
				}
			}

			if( ! contproc ) continue;
		}

		output << "\n\n";
		output << setw(3) << j << "   " << varName << "\n\n";

		for( i = 0 ; i < numAttrs ; i++ ) attrInquire( i, j );
	}


	stat = CDFclose(id);

	return 0;
}
